@@ -3,26 +3,26 @@
 
 #include "ftpcontrolsocket.h"
 
-enum loginStates
-{
-	LOGON_CONNECT,
-	LOGON_WELCOME,
-	LOGON_AUTH_TLS,
-	LOGON_AUTH_SSL,
-	LOGON_AUTH_WAIT,
-	LOGON_SECURITY,
-	LOGON_LOGON,
-	LOGON_SYST,
-	LOGON_FEAT,
-	LOGON_CLNT,
-	LOGON_OPTSUTF8,
-	LOGON_PBSZ,
-	LOGON_PROT,
-	LOGON_OPTSMLST,
-	LOGON_CUSTOMCOMMANDS,
-	LOGON_DONE
+namespace ftpLogonStates {
+enum type {
+        LOGON_CONNECT,
+        LOGON_WELCOME,
+        LOGON_AUTH_TLS,
+        LOGON_AUTH_SSL,
+        LOGON_AUTH_WAIT,
+        LOGON_SECURITY,
+        LOGON_LOGON,
+        LOGON_SYST,
+        LOGON_FEAT,
+        LOGON_CLNT,
+        LOGON_OPTSUTF8,
+        LOGON_PBSZ,
+        LOGON_PROT,
+        LOGON_OPTSMLST,
+        LOGON_CUSTOMCOMMANDS,
+        LOGON_DONE
 };
-
+}
 
 enum class loginCommandType
 {
@@ -73,7 +73,7 @@ private:
 
 	unsigned int customCommandIndex{};
 
-	int neededCommands[LOGON_DONE]{};
+	int neededCommands[ftpLogonStates::LOGON_DONE]{};
 
 	std::deque<t_loginCommand> loginSequence;
 
