@@ -86,8 +86,6 @@ protected:
 };
 }
 
-CEditHandler* CEditHandler::m_pEditHandler = 0;
-
 CEditHandler::CEditHandler(COptionsBase & options)
     : options_(options)
 {
@@ -101,20 +99,6 @@ CEditHandler::CEditHandler(COptionsBase & options)
 #endif
 }
 
-CEditHandler* CEditHandler::Create()
-{
-	if (!m_pEditHandler) {
-		m_pEditHandler = new CEditHandler(*COptions::Get());
-	}
-
-	return m_pEditHandler;
-}
-
-CEditHandler* CEditHandler::Get()
-{
-	return m_pEditHandler;
-}
-
 void CEditHandler::RemoveTemporaryFiles(std::wstring const& temp)
 {
 	wxDir dir(temp);
@@ -250,7 +234,7 @@ std::wstring CEditHandler::GetLocalDirectory()
 	return m_localDir;
 }
 
-void CEditHandler::Release()
+CEditHandler::~CEditHandler()
 {
 	if (m_timer.IsRunning()) {
 		m_timer.Stop();
@@ -277,9 +261,6 @@ void CEditHandler::Release()
 		RemoveAll(true);
 		RemoveTemporaryFilesInSpecificDir(m_localDir);
 	}
-
-	m_pEditHandler = 0;
-	delete this;
 }
 
 CEditHandler::fileState CEditHandler::GetFileState(std::wstring const& fileName) const
@@ -341,7 +322,7 @@ int CEditHandler::GetFileCount(CEditHandler::fileType type, CEditHandler::fileSt
 
 bool CEditHandler::AddFile(CEditHandler::fileType type, std::wstring const& localFile, std::wstring const& remoteFile, CServerPath const& remotePath, Site const& site, int64_t size)
 {
-	if (type == none) {
+	if (type == none || !m_pQueue) {
 		wxBell();
 		return false;
 	}
@@ -881,6 +862,11 @@ bool CEditHandler::UploadFile(std::wstring const& file, bool unedit)
 
 bool CEditHandler::UploadFile(fileType type, std::list<t_fileData>::iterator iter, bool unedit)
 {
+	if (!m_pQueue) {
+		wxBell();
+		return false;
+	}
+
 	wxCHECK(type != none, false);
 
 	if (iter == m_fileDataList[type].end()) {
@@ -909,8 +895,6 @@ bool CEditHandler::UploadFile(fileType type, std::list<t_fileData>::iterator ite
 
 	iter->modificationTime = mtime;
 
-	wxASSERT(m_pQueue);
-
 	std::wstring file;
 	CLocalPath localPath(iter->localFile, &file);
 	if (file.empty()) {
@@ -1345,8 +1329,15 @@ bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CSe
 
 struct CEditHandlerStatusDialog::impl final
 {
+	impl(COptionsBase& options, CEditHandler* edit_handler)
+		: options_(options)
+		, editHandler_(edit_handler)
+	{}
+
 	wxWindow* parent_{};
 
+	COptionsBase& options_;
+
 	wxListCtrlEx* listCtrl_{};
 
 	wxButton* unedit_{};
@@ -1358,8 +1349,8 @@ struct CEditHandlerStatusDialog::impl final
 	std::unique_ptr<CWindowStateManager> windowStateManager_;
 };
 
-CEditHandlerStatusDialog::CEditHandlerStatusDialog(wxWindow* parent)
-	: impl_(std::make_unique<impl>())
+CEditHandlerStatusDialog::CEditHandlerStatusDialog(wxWindow* parent, COptionsBase& options, CEditHandler* edit_handler)
+	: impl_(std::make_unique<impl>(options, edit_handler))
 {
 	impl_->parent_ = parent;
 }
@@ -1373,7 +1364,6 @@ CEditHandlerStatusDialog::~CEditHandlerStatusDialog()
 
 int CEditHandlerStatusDialog::ShowModal()
 {
-	impl_->editHandler_ = CEditHandler::Get();
 	if (!impl_->editHandler_) {
 		return wxID_CANCEL;
 	}
@@ -1503,7 +1493,7 @@ int CEditHandlerStatusDialog::ShowModal()
 	GetSizer()->Fit(this);
 	SetMinClientSize(GetSizer()->GetMinSize());
 
-	impl_->windowStateManager_ = std::make_unique<CWindowStateManager>(static_cast<wxTopLevelWindow*>(this), *COptions::Get());
+	impl_->windowStateManager_ = std::make_unique<CWindowStateManager>(static_cast<wxTopLevelWindow*>(this), impl_->options_);
 	impl_->windowStateManager_->Restore(OPTION_EDITSTATUSDIALOG_SIZE, GetSize());
 
 	SetCtrlState();
