@@ -18,11 +18,14 @@ public:
 	void SetQueue(CQueueView* pQueue) { m_pQueue = pQueue; }
 
 	void StartRecursiveOperation(OperationMode mode, ActiveFilters const& filters, bool immediate = true, bool ignore_links = true);
+
 protected:
 	bool do_start_recursive_operation(OperationMode mode, ActiveFilters const& filters, bool ignore_links) override;
 	void on_listed_directory() override;
+	void on_listing_failed() override;
 
 	void OnListedDirectory();
+	void OnListingFailed();
 
 	bool m_immediate{true};
 	CQueueView* m_pQueue{};
