@@ -14,7 +14,8 @@
 #include "netconfwizard.h"
 #include "Options.h"
 #include "textctrlex.h"
-#include "xrc_helper.h"
+
+#include <wx/statbox.h>
 
 wxDECLARE_EVENT(fzEVT_ON_EXTERNAL_IP_ADDRESS, wxCommandEvent);
 wxDEFINE_EVENT(fzEVT_ON_EXTERNAL_IP_ADDRESS, wxCommandEvent);
@@ -22,7 +23,6 @@ wxDEFINE_EVENT(fzEVT_ON_EXTERNAL_IP_ADDRESS, wxCommandEvent);
 BEGIN_EVENT_TABLE(CNetConfWizard, wxWizard)
 EVT_WIZARD_PAGE_CHANGING(wxID_ANY, CNetConfWizard::OnPageChanging)
 EVT_WIZARD_PAGE_CHANGED(wxID_ANY, CNetConfWizard::OnPageChanged)
-EVT_BUTTON(XRCID("ID_RESTART"), CNetConfWizard::OnRestart)
 EVT_WIZARD_FINISHED(wxID_ANY, CNetConfWizard::OnFinish)
 EVT_TIMER(wxID_ANY, CNetConfWizard::OnTimer)
 EVT_COMMAND(wxID_ANY, fzEVT_ON_EXTERNAL_IP_ADDRESS, CNetConfWizard::OnExternalIPAddress2)
@@ -34,10 +34,37 @@ fztranslate_mark("&Next >");
 fztranslate_mark("< &Back");
 #endif
 
+struct CNetConfWizard::impl
+{
+	wxRadioButton* use_passive_{};
+	wxRadioButton* use_active_{};
+	wxCheckBox* fallback_{};
+
+	wxRadioButton* passive_use_external_{};
+	wxRadioButton* passive_fallback_{};
+
+	wxRadioButton* active_ip_os_{};
+	wxRadioButton* active_ip_provided_{};
+	wxRadioButton* active_ip_resolved_{};
+	wxTextCtrlEx* active_ip_{};
+	wxTextCtrlEx* resolver_{};
+	wxCheckBox* no_external_on_local_{};
+
+	wxRadioButton* active_portmode_any_{};
+	wxRadioButton* active_portmode_specified_{};
+	wxTextCtrlEx* port_min_{};
+	wxTextCtrlEx* port_max_{};
+
+	wxTextCtrl* results_{};
+
+	wxStaticText* summary_[5]{};
+};
+
 CNetConfWizard::CNetConfWizard(wxWindow* parent, CFileZillaEngineContext & engine_context)
 	: fz::event_handler(engine_context.GetEventLoop())
 	, engine_context_(engine_context)
 	, m_parent(parent)
+	, impl_(std::make_unique<impl>())
 {
 	m_timer.SetOwner(this);
 
@@ -58,58 +85,152 @@ bool CNetConfWizard::CreatePages()
 {
 	DialogLayout lay(this);
 
-	InitXrc(L"netconfwizard.xrc");
-	for (int i = 1; i <= 7; ++i) {
-		wxWizardPageSimple* page = new wxWizardPageSimple();
-		if (i == 4) {
-			page->Create(this);
-			auto main = lay.createMain(page, 1);
-			main->Add(new wxStaticText(page, nullID, _("In order to use active mode, FileZilla needs to know your external IP address.")));
+	{
+		wxWizardPageSimple* page = new wxWizardPageSimple(this);
 
-			main->Add(new wxRadioButton(page, XRCID("ID_ACTIVEMODE1"), _("Ask your operating system for the external IP address"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP));
-			main->Add(new wxStaticText(page, nullID, _("This only works if you are not behind a router, else your system would just return your internal address.")), 0, wxLEFT, lay.indent);
-
-			main->Add(new wxRadioButton(page, XRCID("ID_ACTIVEMODE2"), _("Use the following IP address:")));
-			main->Add(new wxTextCtrl(page, XRCID("ID_ACTIVEIP"), wxString(), wxDefaultPosition, wxSize(lay.dlgUnits(60), -1)), 0, wxLEFT, lay.indent);
-			main->Add(new wxStaticText(page, nullID, _("Use this if you're behind a router and have a static external IP address.")), 0, wxLEFT, lay.indent);
-
-			main->Add(new wxRadioButton(page, XRCID("ID_ACTIVEMODE3"), _("Get external IP address from the following URL:")));
-			main->Add(new wxTextCtrl(page, XRCID("ID_ACTIVERESOLVER")), 0, wxLEFT|wxGROW, lay.indent);
-			main->Add(new wxStaticText(page, nullID, _("Default: http://ip.filezilla-project.org/ip.php")), 0, wxLEFT, lay.indent);
-			main->Add(new wxStaticText(page, nullID, _("Use this option if you have a dynamic IP address. FileZilla will contact the above server once each session as soon as you use active mode for the first time. Only the version of FileZilla you're using is submitted to the server.")), 0, wxLEFT, lay.indent);
-
-			main->Add(new wxCheckBox(page, XRCID("ID_NOEXTERNALONLOCAL"), _("Don't use external IP address on &local connections.")));
-		}
-		else if (i == 5) {
-			page->Create(this);
-			auto main = lay.createMain(page, 1);
-			main->Add(new wxStaticText(page, nullID, _("Configure port range")));
-			main->Add(new wxStaticText(page, nullID, _("In active mode, FileZilla has to listen on a port for data transfers. You have to specify which ports FileZilla will use.")));
-
-			main->Add(new wxRadioButton(page, XRCID("ID_ACTIVE_PORTMODE1"), _("Ask operating system for a port."), wxDefaultPosition, wxDefaultSize, wxRB_GROUP));
-			main->Add(new wxStaticText(page, nullID, _("In case you have a router, you will have to forward all available ports, as FileZilla has no influence on the ports your system chooses.")), 0, wxLEFT, lay.indent);
-
-			main->Add(new wxRadioButton(page, XRCID("ID_ACTIVE_PORTMODE2"), _("Use the following port range:")));
-			auto inner = lay.createFlex(3);
-			main->Add(inner, 0, wxLEFT, lay.indent);
-			inner->Add(new wxTextCtrl(page, XRCID("ID_ACTIVE_PORTMIN"), wxString(), wxDefaultPosition, lay.defTextCtrlSize), lay.valign);
-			inner->Add(new wxStaticText(page, nullID, L"-"), lay.valign);
-			inner->Add(new wxTextCtrl(page, XRCID("ID_ACTIVE_PORTMAX"), wxString(), wxDefaultPosition, lay.defTextCtrlSize), lay.valign);
-			main->Add(new wxStaticText(page, nullID, _("All ports in the given range have to be between 1024 and 65535.")), 0, wxLEFT, lay.indent);
-			main->Add(new wxStaticText(page, nullID, _("For reliability you should specify a range of at least 10 ports.")), 0, wxLEFT, lay.indent);
-			main->Add(new wxStaticText(page, nullID, _("If you use a router, make sure all these ports are forwarded to the machine you're running FileZilla on.")), 0, wxLEFT, lay.indent);
-			main->Add(new wxStaticText(page, nullID, _("If you use a firewall, make sure FileZilla is allowed to accept connection on all given ports.")), 0, wxLEFT, lay.indent);
-		}
-		else {
-			bool res = wxXmlResource::Get()->LoadPanel(page, this, wxString::Format(_T("NETCONF_PANEL%d"), i));
-			if (!res) {
-				return false;
-			}
-		}
-		page->Show(false);
-
-		m_pages.push_back(page);
+		auto main = lay.createMain(page, 1);
+		main->Add(new wxStaticText(page, nullID, _("This wizard will help you to properly configure any routers and firewalls you have and tests your configuration.")));
+		main->Add(new wxStaticText(page, nullID, _("Please follow the instructions carefully, as wrong configuration will prevent successful FTP connections.")));
+		main->AddSpacer(0);
+		main->Add(new wxStaticText(page, nullID, _("If you successfully complete the wizard and the final test succeeds, any firewalls and routers you have should be configured properly.")));
+		m_pages.emplace_back(page);
 	}
+	{
+		wxWizardPageSimple* page = new wxWizardPageSimple(this);
+
+		auto main = lay.createMain(page, 1);
+
+		main->Add(new wxStaticText(page, nullID, _("Please select the default transfer mode you would like to use.")));
+		main->Add(new wxStaticText(page, nullID, _("FTP uses secondary connections for data transfers. These additional connections can be established in two ways.")));
+		main->Add(new wxStaticText(page, nullID, _("In passive mode, which is recommended for most users, FileZilla asks the server for a port and connects to the given port. Not much configuration has to be done for this.")));
+		main->Add(new wxStaticText(page, nullID, _("A few servers don't allow passive mode or are misconfigured. In such cases you would have to use active mode. In active mode, FileZilla opens a port and asks the server to connect to it.")));
+
+		main->Add(new wxStaticText(page, nullID, _("Default transfer mode:")));
+		impl_->use_passive_ = new wxRadioButton(page, nullID, _("Passive (recommended)"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
+		main->Add(impl_->use_passive_);
+		impl_->use_active_ = new wxRadioButton(page, nullID, _("&Active"));
+		main->Add(impl_->use_active_);
+		impl_->fallback_ = new wxCheckBox(page, nullID, _("Allow fallback to other transfer mode on failure"));
+		main->Add(impl_->fallback_);
+
+		main->Add(new wxStaticText(page, nullID, _("If the fallback option is enabled, you will be able to connect to misconfigured servers which reject the selected transfer mode.")));
+		main->Add(new wxStaticText(page, nullID, _("On the following pages, the selected transfer mode will be configured, as well as the fallback mode if enabled.")));
+
+		m_pages.emplace_back(page);
+	}
+	{
+		wxWizardPageSimple* page = new wxWizardPageSimple(this);
+
+		auto main = lay.createMain(page, 1);
+
+		main->Add(new wxStaticText(page, nullID, _("Configure passive mode")));
+		main->Add(new wxStaticText(page, nullID, _("Please configure any firewalls and routers so that FileZilla can establish outgoing connection to arbitrary ports.")));
+		main->Add(new wxStaticText(page, nullID, _("Please note that FileZilla has no influence on the port the server chooses. So the server might choose a port which your firewall thinks is only used by trojans or other malware. This might raise a false alarm you can safely discard.")));
+		main->Add(new wxStaticText(page, nullID, _("If the automatic test at the end of this wizard succeeds, but you cannot transfer files to a particular server, the server might be misconfigured and you should try active mode. You can set the transfer mode on a per-server basis in the Site Manager.")));
+		main->Add(new wxStaticText(page, nullID, _("Some misconfigured remote servers which are behind a router, may reply with their local IP address.")));
+
+		impl_->passive_use_external_ = new wxRadioButton(page, nullID, _("Use the server's external IP address instead"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
+		main->Add(impl_->passive_use_external_);
+		impl_->passive_fallback_ = new wxRadioButton(page, nullID, _("Fall back to active mode"));
+		main->Add(impl_->passive_fallback_);
+
+		m_pages.emplace_back(page);
+	}
+	{
+		wxWizardPageSimple* page = new wxWizardPageSimple();
+		page->Create(this);
+		auto main = lay.createMain(page, 1);
+		main->Add(new wxStaticText(page, nullID, _("In order to use active mode, FileZilla needs to know your external IP address.")));
+
+		impl_->active_ip_os_ = new wxRadioButton(page, nullID, _("Ask your operating system for the external IP address"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
+		main->Add(impl_->active_ip_os_);
+		main->Add(new wxStaticText(page, nullID, _("This only works if you are not behind a router, else your system would just return your internal address.")), 0, wxLEFT, lay.indent);
+
+		impl_->active_ip_provided_ = new wxRadioButton(page, nullID, _("Use the following IP address:"));
+		main->Add(impl_->active_ip_provided_);
+		impl_->active_ip_ = new wxTextCtrlEx(page, nullID, wxString(), wxDefaultPosition, wxSize(lay.dlgUnits(60), -1));
+		main->Add(impl_->active_ip_, 0, wxLEFT, lay.indent);
+		main->Add(new wxStaticText(page, nullID, _("Use this if you're behind a router and have a static external IP address.")), 0, wxLEFT, lay.indent);
+
+		impl_->active_ip_resolved_ = new wxRadioButton(page, nullID, _("Get external IP address from the following URL:"));
+		main->Add(impl_->active_ip_resolved_);
+		impl_->resolver_ = new wxTextCtrlEx(page, nullID);
+		main->Add(impl_->resolver_, 0, wxLEFT|wxGROW, lay.indent);
+		main->Add(new wxStaticText(page, nullID, _("Default: http://ip.filezilla-project.org/ip.php")), 0, wxLEFT, lay.indent);
+		main->Add(new wxStaticText(page, nullID, _("Use this option if you have a dynamic IP address. FileZilla will contact the above server once each session as soon as you use active mode for the first time. Only the version of FileZilla you're using is submitted to the server.")), 0, wxLEFT, lay.indent);
+
+		impl_->no_external_on_local_ = new wxCheckBox(page, nullID, _("Don't use external IP address on &local connections."));
+		main->Add(impl_->no_external_on_local_);
+
+		m_pages.emplace_back(page);
+	}
+	{
+		wxWizardPageSimple* page = new wxWizardPageSimple();
+		page->Create(this);
+		auto main = lay.createMain(page, 1);
+		main->Add(new wxStaticText(page, nullID, _("Configure port range")));
+		main->Add(new wxStaticText(page, nullID, _("In active mode, FileZilla has to listen on a port for data transfers. You have to specify which ports FileZilla will use.")));
+
+		impl_->active_portmode_any_ = new wxRadioButton(page, nullID, _("Ask operating system for a port."), wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
+		main->Add(impl_->active_portmode_any_);
+		main->Add(new wxStaticText(page, nullID, _("In case you have a router, you will have to forward all available ports, as FileZilla has no influence on the ports your system chooses.")), 0, wxLEFT, lay.indent);
+
+		impl_->active_portmode_specified_ = new wxRadioButton(page, nullID, _("Use the following port range:"));
+		main->Add(impl_->active_portmode_specified_);
+		auto inner = lay.createFlex(3);
+		main->Add(inner, 0, wxLEFT, lay.indent);
+		impl_->port_min_ = new wxTextCtrlEx(page, nullID, wxString(), wxDefaultPosition, lay.defTextCtrlSize);
+		inner->Add(impl_->port_min_, lay.valign);
+		inner->Add(new wxStaticText(page, nullID, L"-"), lay.valign);
+		impl_->port_max_ = new wxTextCtrlEx(page, nullID, wxString(), wxDefaultPosition, lay.defTextCtrlSize);
+		inner->Add(impl_->port_max_, lay.valign);
+		main->Add(new wxStaticText(page, nullID, _("All ports in the given range have to be between 1024 and 65535.")), 0, wxLEFT, lay.indent);
+		main->Add(new wxStaticText(page, nullID, _("For reliability you should specify a range of at least 10 ports.")), 0, wxLEFT, lay.indent);
+		main->Add(new wxStaticText(page, nullID, _("If you use a router, make sure all these ports are forwarded to the machine you're running FileZilla on.")), 0, wxLEFT, lay.indent);
+		main->Add(new wxStaticText(page, nullID, _("If you use a firewall, make sure FileZilla is allowed to accept connection on all given ports.")), 0, wxLEFT, lay.indent);
+
+		m_pages.emplace_back(page);
+	}
+	{
+		wxWizardPageSimple* page = new wxWizardPageSimple();
+		page->Create(this);
+		auto main = lay.createMain(page, 1);
+		main->AddGrowableCol(0);
+		main->AddGrowableRow(5);
+		main->Add(new wxStaticText(page, nullID, _("Configuration test")));
+		main->Add(new wxStaticText(page, nullID, _("At this point, all required information has been gathered.")));
+		main->Add(new wxStaticText(page, nullID, _("FileZilla can now test your configuration to ensure everything is configured properly.")));
+		main->Add(new wxStaticText(page, nullID, _("If you click on test, FileZilla will connect to probe.filezilla-project.org and will perform some simple tests.")));
+		main->Add(new wxStaticText(page, nullID, _("If you get any errors, your configuration is not correct.")));
+		auto [box, inner] = lay.createStatBox(main, _("Test results"), 1);
+		inner->AddGrowableCol(0);
+		inner->AddGrowableRow(0);
+		impl_->results_ = new wxTextCtrl(box, nullID, wxString(), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY);
+		inner->Add(impl_->results_, 1, wxGROW);
+		m_pages.emplace_back(page);
+	}
+	{
+		wxWizardPageSimple* page = new wxWizardPageSimple();
+		page->Create(this);
+		auto main = lay.createMain(page, 1);
+
+		main->Add(new wxStaticText(page, nullID, _("Summary of test results:")));
+		for (size_t i = 0; i < 5; ++i) {
+			impl_->summary_[i] = new wxStaticText(page, nullID, wxString());
+			main->Add(impl_->summary_[i]);
+		}
+
+		main->AddSpacer(0);
+		main->AddGrowableCol(0);
+		main->AddGrowableRow(6);
+		auto restart = new wxButton(page, nullID, _("&Restart Wizard"));
+		main->Add(restart, 0, wxALIGN_BOTTOM | wxALIGN_RIGHT);
+		restart->Bind(wxEVT_BUTTON, [&](wxCommandEvent&){ OnRestart(); });
+
+
+		m_pages.emplace_back(page);
+	}
+
 	for (unsigned int i = 0; i < (m_pages.size() - 1); ++i) {
 		m_pages[i]->Chain(m_pages[i], m_pages[i + 1]);
 	}
@@ -118,7 +239,7 @@ bool CNetConfWizard::CreatePages()
 
 bool CNetConfWizard::Load()
 {
-	if (!Create(m_parent, wxID_ANY, _("Firewall and router configuration wizard"), wxNullBitmap, wxPoint(0, 0))) {
+	if (!Create(m_parent, nullID, _("Firewall and router configuration wizard"), wxNullBitmap, wxPoint(0, 0))) {
 		return false;
 	}
 
@@ -144,53 +265,53 @@ bool CNetConfWizard::Load()
 	{
 	default:
 	case 1:
-		XRCCTRL(*this, "ID_PASSIVE", wxRadioButton)->SetValue(true);
+		impl_->use_passive_->SetValue(true);
 		break;
 	case 0:
-		XRCCTRL(*this, "ID_ACTIVE", wxRadioButton)->SetValue(true);
+		impl_->use_active_->SetValue(true);
 		break;
 	}
 
-	XRCCTRL(*this, "ID_FALLBACK", wxCheckBox)->SetValue(engine_context_.GetOptions().get_int(OPTION_ALLOW_TRANSFERMODEFALLBACK) != 0);
+	impl_->fallback_->SetValue(engine_context_.GetOptions().get_int(OPTION_ALLOW_TRANSFERMODEFALLBACK) != 0);
 
 	switch (engine_context_.GetOptions().get_int(OPTION_PASVREPLYFALLBACKMODE))
 	{
 	default:
 	case 0:
-		XRCCTRL(*this, "ID_PASSIVE_FALLBACK1", wxRadioButton)->SetValue(true);
+		impl_->passive_use_external_->SetValue(true);
 		break;
 	case 1:
-		XRCCTRL(*this, "ID_PASSIVE_FALLBACK2", wxRadioButton)->SetValue(true);
+		impl_->passive_fallback_->SetValue(true);
 		break;
 	}
 	switch (engine_context_.GetOptions().get_int(OPTION_EXTERNALIPMODE))
 	{
 	default:
 	case 0:
-		XRCCTRL(*this, "ID_ACTIVEMODE1", wxRadioButton)->SetValue(true);
+		impl_->active_ip_os_->SetValue(true);
 		break;
 	case 1:
-		XRCCTRL(*this, "ID_ACTIVEMODE2", wxRadioButton)->SetValue(true);
+		impl_->active_ip_provided_->SetValue(true);
 		break;
 	case 2:
-		XRCCTRL(*this, "ID_ACTIVEMODE3", wxRadioButton)->SetValue(true);
+		impl_->active_ip_resolved_->SetValue(true);
 		break;
 	}
 	switch (engine_context_.GetOptions().get_int(OPTION_LIMITPORTS))
 	{
 	default:
 	case 0:
-		XRCCTRL(*this, "ID_ACTIVE_PORTMODE1", wxRadioButton)->SetValue(true);
+		impl_->active_portmode_any_->SetValue(true);
 		break;
 	case 1:
-		XRCCTRL(*this, "ID_ACTIVE_PORTMODE2", wxRadioButton)->SetValue(true);
+		impl_->active_portmode_specified_->SetValue(true);
 		break;
 	}
-	XRCCTRL(*this, "ID_ACTIVE_PORTMIN", wxTextCtrl)->SetValue(wxString::Format(_T("%d"), engine_context_.GetOptions().get_int(OPTION_LIMITPORTS_LOW)));
-	XRCCTRL(*this, "ID_ACTIVE_PORTMAX", wxTextCtrl)->SetValue(wxString::Format(_T("%d"), engine_context_.GetOptions().get_int(OPTION_LIMITPORTS_HIGH)));
-	XRCCTRL(*this, "ID_ACTIVEIP", wxTextCtrl)->SetValue(engine_context_.GetOptions().get_string(OPTION_EXTERNALIP));
-	XRCCTRL(*this, "ID_ACTIVERESOLVER", wxTextCtrl)->SetValue(engine_context_.GetOptions().get_string(OPTION_EXTERNALIPRESOLVER));
-	XRCCTRL(*this, "ID_NOEXTERNALONLOCAL", wxCheckBox)->SetValue(engine_context_.GetOptions().get_int(OPTION_NOEXTERNALONLOCAL) != 0);
+	impl_->port_min_->SetValue(wxString::Format(_T("%d"), engine_context_.GetOptions().get_int(OPTION_LIMITPORTS_LOW)));
+	impl_->port_max_->SetValue(wxString::Format(_T("%d"), engine_context_.GetOptions().get_int(OPTION_LIMITPORTS_HIGH)));
+	impl_->active_ip_->SetValue(engine_context_.GetOptions().get_string(OPTION_EXTERNALIP));
+	impl_->resolver_->SetValue(engine_context_.GetOptions().get_string(OPTION_EXTERNALIPRESOLVER));
+	impl_->no_external_on_local_->SetValue(engine_context_.GetOptions().get_int(OPTION_NOEXTERNALONLOCAL) != 0);
 
 	return true;
 }
@@ -203,48 +324,43 @@ bool CNetConfWizard::Run()
 void CNetConfWizard::OnPageChanging(wxWizardEvent& event)
 {
 	if (event.GetPage() == m_pages[3]) {
-		int mode = XRCCTRL(*this, "ID_ACTIVEMODE1", wxRadioButton)->GetValue() ? 0 : (XRCCTRL(*this, "ID_ACTIVEMODE2", wxRadioButton)->GetValue() ? 1 : 2);
+		int mode = impl_->active_ip_os_->GetValue() ? 0 : (impl_->active_ip_provided_->GetValue() ? 1 : 2);
 		if (mode == 1) {
-			wxTextCtrl* control = XRCCTRL(*this, "ID_ACTIVEIP", wxTextCtrl);
-			std::wstring ip = control->GetValue().ToStdWstring();
+			std::wstring ip = impl_->active_ip_->GetValue().ToStdWstring();
 			if (ip.empty()) {
 				wxMessageBoxEx(_("Please enter your external IP address"));
-				control->SetFocus();
+				impl_->active_ip_->SetFocus();
 				event.Veto();
 				return;
 			}
 			if (fz::get_address_type(ip) != fz::address_type::ipv4) {
 				wxMessageBoxEx(_("You have to enter a valid IPv4 address."));
-				control->SetFocus();
+				impl_->active_ip_->SetFocus();
 				event.Veto();
 				return;
 			}
 		}
 		else if (mode == 2) {
-			wxTextCtrl* pResolver = XRCCTRL(*this, "ID_ACTIVERESOLVER", wxTextCtrl);
-			wxString address = pResolver->GetValue();
+			std::wstring address = impl_->resolver_->GetValue().ToStdWstring();
 			if (address.empty()) {
 				wxMessageBoxEx(_("Please enter an URL where to get your external address from"));
-				pResolver->SetFocus();
+				impl_->resolver_->SetFocus();
 				event.Veto();
 				return;
 			}
 		}
 	}
 	else if (event.GetPage() == m_pages[4]) {
-		int mode = XRCCTRL(*this, "ID_ACTIVE_PORTMODE1", wxRadioButton)->GetValue() ? 0 : 1;
-		if (mode) {
-			wxTextCtrl* pPortMin = XRCCTRL(*this, "ID_ACTIVE_PORTMIN", wxTextCtrl);
-			wxTextCtrl* pPortMax = XRCCTRL(*this, "ID_ACTIVE_PORTMAX", wxTextCtrl);
-			wxString portMin = pPortMin->GetValue();
-			wxString portMax = pPortMax->GetValue();
+		if (impl_->active_portmode_specified_->GetValue()) {
+			wxString portMin = impl_->port_min_->GetValue();
+			wxString portMax = impl_->port_max_->GetValue();
 
 			long min = 0, max = 0;
 			if (!portMin.ToLong(&min) || !portMax.ToLong(&max) ||
 				min < 1024 || max > 65535 || min > max)
 			{
 				wxMessageBoxEx(_("Please enter a valid portrange."));
-				pPortMin->SetFocus();
+				impl_->port_min_->SetFocus();
 				event.Veto();
 				return;
 			}
@@ -599,7 +715,7 @@ void CNetConfWizard::ParseResponse(const char* line)
 
 void CNetConfWizard::PrintMessage(std::wstring const& msg, int)
 {
-	XRCCTRL(*this, "ID_RESULTS", wxTextCtrl)->AppendText(msg + L"\n");
+	impl_->results_->AppendText(msg + L"\n");
 }
 
 void CNetConfWizard::CloseSocket()
@@ -650,8 +766,8 @@ void CNetConfWizard::CloseSocket()
 			text[1] = _("Please update your firewall and make sure your router is using the latest available firmware. Furthermore, your router has to be configured properly. You will have to use manual port forwarding. Don't run your router in the so called 'DMZ mode' or 'game mode'. Things like protocol inspection or protocol specific 'fixups' have to be disabled");
 			text[2] = _("If this problem stays, please contact your router manufacturer.");
 			text[3] = _("Unless this problem gets fixed, active mode FTP will not work and passive mode has to be used.");
-			if (XRCCTRL(*this, "ID_ACTIVE", wxRadioButton)->GetValue()) {
-				XRCCTRL(*this, "ID_PASSIVE", wxRadioButton)->SetValue(true);
+			if (impl_->use_active_->GetValue()) {
+				impl_->use_passive_->SetValue(true);
 				text[3] += _T(" ");
 				text[3] += _("Passive mode has been set as default transfer mode.");
 			}
@@ -662,8 +778,8 @@ void CNetConfWizard::CloseSocket()
 			text[2] = _("Please make sure your router is using the latest available firmware. Furthermore, your router has to be configured properly. You will have to use manual port forwarding. Don't run your router in the so called 'DMZ mode' or 'game mode'.");
 			text[3] = _("If your router keeps changing the IP address, please contact your router manufacturer.");
 			text[4] = _("Unless these problems get fixed, active mode FTP will not work and passive mode has to be used.");
-			if (XRCCTRL(*this, "ID_ACTIVE", wxRadioButton)->GetValue()) {
-				XRCCTRL(*this, "ID_PASSIVE", wxRadioButton)->SetValue(true);
+			if (impl_->use_active_->GetValue()) {
+				impl_->use_passive_->SetValue(true);
 				text[4] += _T(" ");
 				text[4] += _("Passive mode has been set as default transfer mode.");
 			}
@@ -672,8 +788,8 @@ void CNetConfWizard::CloseSocket()
 			text[0] = _("Active mode FTP test failed. FileZilla does not know the correct external IP address.");
 			text[1] = _("Please enter your external IP address on the active mode page of this wizard. In case you have a dynamic address or don't know your external address, use the external resolver option.");
 			text[2] = _("Unless these problems get fixed, active mode FTP will not work and passive mode has to be used.");
-			if (XRCCTRL(*this, "ID_ACTIVE", wxRadioButton)->GetValue()) {
-				XRCCTRL(*this, "ID_PASSIVE", wxRadioButton)->SetValue(true);
+			if (impl_->use_active_->GetValue()) {
+				impl_->use_passive_->SetValue(true);
 				text[2] += _T(" ");
 				text[2] += _("Passive mode has been set as default transfer mode.");
 			}
@@ -681,7 +797,7 @@ void CNetConfWizard::CloseSocket()
 		case externalfailed:
 			text[0] = _("Failed to retrieve the external IP address.");
 			text[1] = _("Please make sure FileZilla is allowed to establish outgoing connections and make sure you typed the address of the address resolver correctly.");
-			text[2] = wxString::Format(_("The address you entered was: %s"), XRCCTRL(*this, "ID_ACTIVERESOLVER", wxTextCtrl)->GetValue());
+			text[2] = wxString::Format(_("The address you entered was: %s"), impl_->resolver_->GetValue());
 			break;
 		case datatainted:
 			text[0] = _("Transferred data got tainted.");
@@ -692,19 +808,14 @@ void CNetConfWizard::CloseSocket()
 		}
 	}
 	for (unsigned int i = 0; i < 5; ++i) {
-		wxString name = wxString::Format(_T("ID_SUMMARY%d"), i + 1);
-		int id = wxXmlResource::GetXRCID(name);
-		auto ctrl = dynamic_cast<wxStaticText*>(FindWindowById(id, this));
-		if (ctrl) {
-			ctrl->SetLabel(text[i]);
-		}
+		impl_->summary_[i]->SetLabel(text[i]);
 	}
 	m_pages[6]->GetSizer()->Layout();
 	m_pages[6]->GetSizer()->Fit(m_pages[6]);
 	wxGetApp().GetWrapEngine()->WrapRecursive(m_pages[6], m_pages[6]->GetSizer(), wxGetApp().GetWrapEngine()->GetWidthFromCache("Netconf"));
 
 	// Focus one so enter key hits finish and not the restart button by default
-	XRCCTRL(*this, "ID_SUMMARY1", wxStaticText)->SetFocus();
+	impl_->summary_[0]->SetFocus();
 
 	socket_.reset();
 
@@ -743,7 +854,7 @@ std::wstring CNetConfWizard::GetExternalIPAddress()
 
 	wxASSERT(socket_);
 
-	int mode = XRCCTRL(*this, "ID_ACTIVEMODE1", wxRadioButton)->GetValue() ? 0 : (XRCCTRL(*this, "ID_ACTIVEMODE2", wxRadioButton)->GetValue() ? 1 : 2);
+	int mode = impl_->active_ip_os_->GetValue() ? 0 : (impl_->active_ip_provided_->GetValue() ? 1 : 2);
 	if (!mode) {
 		ret = fz::to_wstring_from_utf8(socket_->local_ip());
 		if (ret.empty()) {
@@ -752,13 +863,11 @@ std::wstring CNetConfWizard::GetExternalIPAddress()
 		}
 	}
 	else if (mode == 1) {
-		wxTextCtrl* control = XRCCTRL(*this, "ID_ACTIVEIP", wxTextCtrl);
-		ret = control->GetValue().ToStdWstring();
+		ret = impl_->active_ip_->GetValue().ToStdWstring();
 	}
 	else if (mode == 2) {
 		if (!m_pIPResolver) {
-			wxTextCtrl* pResolver = XRCCTRL(*this, "ID_ACTIVERESOLVER", wxTextCtrl);
-			std::wstring address = pResolver->GetValue().ToStdWstring();
+			std::wstring address = impl_->resolver_->GetValue().ToStdWstring();
 
 			PrintMessage(fz::sprintf(fztranslate("Retrieving external IP address from %s"), address), 0);
 
@@ -872,7 +981,7 @@ void CNetConfWizard::SendNextCommand()
 	}
 }
 
-void CNetConfWizard::OnRestart(wxCommandEvent&)
+void CNetConfWizard::OnRestart()
 {
 	ResetTest();
 	ShowPage(m_pages[0], false);
@@ -893,7 +1002,7 @@ void CNetConfWizard::ResetTest()
 	gotListReply = false;
 
 	if (!m_pages.empty()) {
-		XRCCTRL(*this, "ID_RESULTS", wxTextCtrl)->SetLabel(_T(""));
+		impl_->results_->ChangeValue(wxString());
 	}
 }
 
@@ -905,27 +1014,27 @@ void CNetConfWizard::OnFinish(wxWizardEvent&)
 		}
 	}
 
-	engine_context_.GetOptions().set(OPTION_USEPASV, XRCCTRL(*this, "ID_PASSIVE", wxRadioButton)->GetValue() ? 1 : 0);
-	engine_context_.GetOptions().set(OPTION_ALLOW_TRANSFERMODEFALLBACK, XRCCTRL(*this, "ID_FALLBACK", wxCheckBox)->GetValue() ? 1 : 0);
+	engine_context_.GetOptions().set(OPTION_USEPASV, impl_->use_passive_->GetValue());
+	engine_context_.GetOptions().set(OPTION_ALLOW_TRANSFERMODEFALLBACK, impl_->fallback_->GetValue());
 
-	engine_context_.GetOptions().set(OPTION_PASVREPLYFALLBACKMODE, XRCCTRL(*this, "ID_PASSIVE_FALLBACK1", wxRadioButton)->GetValue() ? 0 : 1);
+	engine_context_.GetOptions().set(OPTION_PASVREPLYFALLBACKMODE, impl_->passive_use_external_->GetValue() ? 0 : 1);
 
-	if (XRCCTRL(*this, "ID_ACTIVEMODE1", wxRadioButton)->GetValue()) {
+	if (impl_->active_ip_os_->GetValue()) {
 		engine_context_.GetOptions().set(OPTION_EXTERNALIPMODE, 0);
 	}
 	else {
-		engine_context_.GetOptions().set(OPTION_EXTERNALIPMODE, XRCCTRL(*this, "ID_ACTIVEMODE2", wxRadioButton)->GetValue() ? 1 : 2);
+		engine_context_.GetOptions().set(OPTION_EXTERNALIPMODE, impl_->active_ip_provided_->GetValue() ? 1 : 2);
 	}
 
-	engine_context_.GetOptions().set(OPTION_LIMITPORTS, XRCCTRL(*this, "ID_ACTIVE_PORTMODE1", wxRadioButton)->GetValue() ? 0 : 1);
+	engine_context_.GetOptions().set(OPTION_LIMITPORTS, impl_->active_portmode_any_->GetValue() ? 0 : 1);
 
 	long tmp;
-	XRCCTRL(*this, "ID_ACTIVE_PORTMIN", wxTextCtrl)->GetValue().ToLong(&tmp); engine_context_.GetOptions().set(OPTION_LIMITPORTS_LOW, tmp);
-	XRCCTRL(*this, "ID_ACTIVE_PORTMAX", wxTextCtrl)->GetValue().ToLong(&tmp); engine_context_.GetOptions().set(OPTION_LIMITPORTS_HIGH, tmp);
+	impl_->port_min_->GetValue().ToLong(&tmp); engine_context_.GetOptions().set(OPTION_LIMITPORTS_LOW, tmp);
+	impl_->port_max_->GetValue().ToLong(&tmp); engine_context_.GetOptions().set(OPTION_LIMITPORTS_HIGH, tmp);
 
-	engine_context_.GetOptions().set(OPTION_EXTERNALIP, XRCCTRL(*this, "ID_ACTIVEIP", wxTextCtrl)->GetValue().ToStdWstring());
-	engine_context_.GetOptions().set(OPTION_EXTERNALIPRESOLVER, XRCCTRL(*this, "ID_ACTIVERESOLVER", wxTextCtrl)->GetValue().ToStdWstring());
-	engine_context_.GetOptions().set(OPTION_NOEXTERNALONLOCAL, XRCCTRL(*this, "ID_NOEXTERNALONLOCAL", wxCheckBox)->GetValue());
+	engine_context_.GetOptions().set(OPTION_EXTERNALIP, impl_->active_ip_->GetValue().ToStdWstring());
+	engine_context_.GetOptions().set(OPTION_EXTERNALIPRESOLVER, impl_->resolver_->GetValue().ToStdWstring());
+	engine_context_.GetOptions().set(OPTION_NOEXTERNALONLOCAL, impl_->no_external_on_local_->GetValue());
 }
 
 int CNetConfWizard::CreateListenSocket()
@@ -934,14 +1043,14 @@ int CNetConfWizard::CreateListenSocket()
 		return 0;
 	}
 
-	if (XRCCTRL(*this, "ID_ACTIVE_PORTMODE1", wxRadioButton)->GetValue()) {
+	if (impl_->active_portmode_any_->GetValue()) {
 		return CreateListenSocket(0);
 	}
 	else {
 		long low;
 		long high;
-		XRCCTRL(*this, "ID_ACTIVE_PORTMIN", wxTextCtrl)->GetValue().ToLong(&low);
-		XRCCTRL(*this, "ID_ACTIVE_PORTMAX", wxTextCtrl)->GetValue().ToLong(&high);
+		impl_->port_min_->GetValue().ToLong(&low);
+		impl_->port_max_->GetValue().ToLong(&high);
 
 		int mid = fz::random_number(low, high);
 		wxASSERT(mid >= low && mid <= high);
